#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <imsls.h>

void l_printLSD(int n1, int *equalMeans, float *means);
void l_printLSD2Table(int n1, int n2, int* equalMeans, float *means);
void l_printLSD3Table(int n1, int n2, int n3, int* equalMeans,
                      float *means);

int main()
{
    char **anova_row_labels;
    char *col_labels[] = {" ", "\nID", "\nDF", "\nSSQ", 
        "Mean\nsquares", "\nF", "\np-value"};
    int i, j, k, l, page_width = 132;

    int n = 24;            /* Total number of observations */
    int n_locations = 1;   /* Number of locations */
    int n_strip_a = 2;     /* Number of Factor A strip-plots within a
                           location */
    int n_strip_b = 2;     /* Number of Factor B strip-plots within a
                           location */
    int n_split   = 2;     /* Number of split-plots within each Factor
                           A strip-plot */
    int block[]={
        1, 1, 1, 1, 1, 1, 1, 1,
        2, 2, 2, 2, 2, 2, 2, 2,
        3, 3, 3, 3, 3, 3, 3, 3
    };
    int strip_a[]={
        1, 1, 1, 1, 2, 2, 2, 2, 
        1, 1, 1, 1, 2, 2, 2, 2,
        1, 1, 1, 1, 2, 2, 2, 2
    };
    int strip_b[]={
        1, 1, 2, 2, 1, 1, 2, 2, 
        1, 1, 2, 2, 1, 1, 2, 2,  
        1, 1, 2, 2, 1, 1, 2, 2
    };
    int split[]={
        1, 2, 1, 2, 1, 2, 1, 2, 
        1, 2, 1, 2, 1, 2, 1, 2,  
        1, 2, 1, 2, 1, 2, 1, 2
    };
    float y[] ={
        30.0, 40.0, 38.9, 38.2,
        41.8, 52.2, 54.8, 58.2,
        20.5, 26.9, 21.4, 25.1,
        26.4, 36.7, 28.9, 35.9,
        21.0, 25.4, 24.0, 23.3,
        34.4, 41.0, 33.0, 34.9
    };
    float alpha = 0.05;
    float grand_mean = 0;
    float *cv;
    float *aov;
    float *treatment_means;
    float *strip_plot_a_means;
    float *strip_plot_b_means;
    float *split_plot_means;
    float *strip_a_split_plot_means;
    float *strip_b_split_plot_means;
    float *strip_plot_ab_means;
    float *std_err;
    int   *equal_means;

    aov = imsls_f_strip_split_plot(n, n_locations, n_strip_a,
        n_strip_b, n_split, block, strip_a, strip_b, split, y, 
        IMSLS_GRAND_MEAN, &grand_mean, 
        IMSLS_CV, &cv,
        IMSLS_TREATMENT_MEANS,  &treatment_means, 
        IMSLS_STRIP_PLOT_A_MEANS, &strip_plot_a_means, 
        IMSLS_STRIP_PLOT_B_MEANS, &strip_plot_b_means, 
        IMSLS_SPLIT_PLOT_MEANS, &split_plot_means,
        IMSLS_STRIP_PLOT_A_SPLIT_PLOT_MEANS, &strip_a_split_plot_means,
        IMSLS_STRIP_PLOT_B_SPLIT_PLOT_MEANS, &strip_b_split_plot_means,
        IMSLS_STRIP_PLOT_AB_MEANS, &strip_plot_ab_means,
        IMSLS_STD_ERRORS, &std_err,
        IMSLS_ANOVA_ROW_LABELS, &anova_row_labels,
        0);

    /* Output results. */
    imsls_page(IMSLS_SET_PAGE_WIDTH, &page_width);
    /* Print ANOVA table, without first column. */
    imsls_f_write_matrix("   *** ANALYSIS OF VARIANCE TABLE ***", 
        22, 6, aov, 
        IMSLS_WRITE_FORMAT, "%3.0f%3.0f%8.2f%7.2f%7.2f%7.3f",
        IMSLS_ROW_LABELS, anova_row_labels,
        IMSLS_COL_LABELS, col_labels,
        0);

    /* 
    * Print the various means.
    */
    printf("\nGrand mean: %f\n\n", grand_mean);
    printf("Coefficient of Variation\n");
    printf("  Strip-Plot A:      %9.4f\n", cv[0]);
    printf("  Strip-Plot B:      %9.4f\n", cv[1]);
    printf("  Split-Plot:        %9.4f\n\n", cv[2]);
    l = 0;

    /* 
    * Print the Treatment Means.
    */
    printf("\n\n**********************************************");
    printf("***************");
    printf("\nTreatment Means\n");
    for (i=0; i < n_strip_a; i++){
        for(j=0; j < n_strip_b; j++){
            for(k=0; k < n_split; k++){
                printf("treatment[%d][%d][%d]   %9.4f \n", 
                    i+1, j+1, k+1, treatment_means[l++]);
            }
        }
    }
    printf("\nStandard Error for Comparing Two Treatment Means: ");
    printf("%f \n(df=%f)\n", std_err[9], std_err[19]);
    equal_means = imsls_f_multiple_comparisons(
        n_strip_a*n_strip_b*n_split,  
        treatment_means, (int) std_err[19],
        std_err[9]/sqrt(2.0), 
        IMSLS_LSD,
        IMSLS_ALPHA, alpha,
        0);
    l_printLSD3Table(n_strip_a, n_strip_b, n_split, equal_means,
        treatment_means);

    /* 
    * Print the Strip-plot A Means. 
    */
    printf("\n\n*****************************************************");
    printf("********");
    imsls_f_write_matrix("Strip-plot A Means", n_strip_a, 1,
        strip_plot_a_means, 0);
    printf("\nStandard Error for Comparing Two Strip-Plot A Means: ");
    printf("%f \n(df=%f)\n",
        std_err[0], std_err[10]);
    equal_means = imsls_f_multiple_comparisons(n_strip_a,
        strip_plot_a_means, (int) std_err[10], std_err[0]/sqrt(2.0), 
        IMSLS_LSD,
        IMSLS_ALPHA, alpha,
        0);
    l_printLSD(n_strip_a, equal_means, strip_plot_a_means);

    /* 
    * Print Strip-plot B Means. 
    */
    printf("\n\n****************************************************");
    printf("*********");
    imsls_f_write_matrix("Strip-plot B Means", n_strip_b, 1,
        strip_plot_b_means, 0);
    printf("\nStandard Error for Comparing Two Strip-Plot B Means: ");
    printf("%f \n(df=%f)\n", std_err[1], std_err[11]);
    equal_means = imsls_f_multiple_comparisons(n_strip_b,
        strip_plot_b_means, 
        (int) std_err[11], std_err[1]/sqrt(2.0), 
        IMSLS_LSD,
        IMSLS_ALPHA, alpha,
        0);
    l_printLSD(n_strip_b, equal_means, strip_plot_b_means);

    /* 
    * Print the Split-plot Means. 
    */
    printf("\n\n****************************************************");
    printf("*********");
    imsls_f_write_matrix("Split-plot Means", n_split, 1,
        split_plot_means, 0);
    printf("\nStandard Error for Comparing Two Split-Plot Means: ");
    printf("%f \n(df=%f)\n", std_err[2], std_err[12]);
    equal_means = imsls_f_multiple_comparisons(n_split,
        split_plot_means, 
        (int) std_err[12], std_err[2]/sqrt(2.0), 
        IMSLS_LSD,
        IMSLS_ALPHA, alpha,
        0);
    l_printLSD(n_split, equal_means, split_plot_means);

    /* 
    * Print the Strip-plot A by Split-plot Means. 
    */
    printf("\n\n***************************************************");
    printf("**********");
    imsls_f_write_matrix("Strip-plot A by Split-plot Means",
        n_strip_a, n_split, strip_a_split_plot_means, 0);
    printf("\nStandard Error for Comparing Two Means: %f \n(df=%f)\n",
        std_err[3], std_err[13]);
    equal_means = imsls_f_multiple_comparisons(n_strip_a*n_split, 
        strip_a_split_plot_means, 
        (int) std_err[13],
        std_err[3]/sqrt(2.0), 
        IMSLS_LSD,
        IMSLS_ALPHA, alpha,
        0);
    l_printLSD2Table(n_strip_a, n_split, equal_means,
        strip_a_split_plot_means);

    /* 
    * Print the Strip-plot A by Strip-plot B Means. 
    */
    printf("\n\n****************************************************");
    printf("*********");
    imsls_f_write_matrix("Strip-plot A by Strip-plot B Means",
        n_strip_a, n_strip_b, strip_plot_ab_means, 0);
    printf("\nStandard Error for Comparing Two Means: %f \n(df=%f)\n",
        std_err[4], std_err[14]);
    equal_means = imsls_f_multiple_comparisons(n_strip_a*n_strip_b, 
        strip_plot_ab_means, (int) std_err[14],
        std_err[4]/sqrt(2.0), 
        IMSLS_LSD,
        IMSLS_ALPHA, alpha,
        0);
    l_printLSD2Table(n_strip_a, n_strip_b, equal_means,
        strip_plot_ab_means);

    /* 
    * Print the Strip-Plot B by Split-Plot Means. 
    */
    printf("\n\n****************************************************");
    printf("*********");
    imsls_f_write_matrix("Strip-Plot B by Split-Plot Means", n_strip_b,
        n_split, strip_b_split_plot_means, 0);
    printf("\nStandard Error for Comparing Two Means: %f \n(df=%f)\n",
        std_err[5], std_err[15]);
    equal_means = imsls_f_multiple_comparisons(n_strip_b*n_split, 
        strip_b_split_plot_means, 
        (int) std_err[15], std_err[5]/sqrt(2.0), 
        IMSLS_LSD,
        IMSLS_ALPHA, alpha,
        0);
    l_printLSD2Table(n_strip_b, n_split, equal_means,
        strip_b_split_plot_means);

}
/* 
* Local functions to output  results of means comparisons.
*/
void l_printLSD(int n, int *equalMeans, float *means){
    float x=0.0;
    int i, j, k;
    int iSwitch;
    int *idx;

    idx = (int *) malloc(n * sizeof (int));

    for (k=0; k < n; k++) {
        idx[k] =k+1;
    }        
    /* Sort means in ascending order*/
    iSwitch=1;
    while (iSwitch != 0){
        iSwitch = 0;
        for (i = 0; i < n-1; i++){
            if (means[i] > means[i+1]){
                iSwitch = 1;
                x = means[i];
                means[i] = means[i+1];
                means[i+1] = x;
                j = idx[i];
                idx[i] = idx[i+1];
                idx[i+1] = j;
            }
        }
    }
    printf("[group] \t  Mean \t\tLSD Grouping \n");
    for (i=0; i < n; i++){
        printf("  [%d] \t\t%f", idx[i], means[i]);

        for (j=1; j < i+1; j++){
            if(equalMeans[j-1] >= i+2-j){
                printf("\t  *");
            }else{ 
                if(equalMeans[j-1]>=0) printf("\t");
            }
        }
        if (i < n-1 && equalMeans[i]>0) printf("\t  *");
        printf("\n");
    }
    imsls_free(idx);
    return;

}
void l_printLSD2Table(int n1, int n2, int *equalMeans, float *means){
    float x=0.0;
    int i, j, k, n;
    int iSwitch;
    int *idx;
    n = n1*n2;

    idx = (int *) malloc(2*n * sizeof (int));
    i = 1;
    j = 1;
    for (k=0; k < n; k++) {
        idx[2*k]   = i;
        idx[2*k+1] = j++;
        if (j > n2){
            j = 1;
            i++;
        }
    }

    /* sort means in ascending order*/

    iSwitch=1;
    while (iSwitch != 0){
        iSwitch = 0;
        for (i = 0; i < n-1; i++){
            if (means[i] > means[i+1]){
                iSwitch = 1;
                x = means[i];
                means[i] = means[i+1];
                means[i+1] = x;
                j = idx[2*i];
                idx[2*i] = idx[2*(i+1)];
                idx[2*(i+1)] = j;
                j = idx[2*i+1];
                idx[2*i+1] = idx[2*(i+1)+1];
                idx[2*(i+1)+1] = j;
            }
        }
    }
    printf("[A][B] \tMean \t\tLSD Grouping \n");
    for (i=0; i < n; i++){
        printf("[%d][%d] \t%f", idx[2*i], idx[2*i+1],means[i]);

        for (j=1; j < i+1; j++){
            if(equalMeans[j-1] >= i+2-j){
                printf("\t*");
            }else{ 
                if(equalMeans[j-1]>0) printf("\t");
            }
        }
        if (i < n-1 && equalMeans[i]>0) printf("\t*");
        printf("\n");
    }
    imsls_free(idx);
    idx = NULL;
    return;

}
void l_printLSD3Table(int n1, int n2, int n3, int *equalMeans,
                      float *means){
    float x=0.0;
    int i, j, k, m, n;
    int iSwitch;
    int *idx;
    n = n1*n2*n3;

    idx = (int *) malloc(3*n * sizeof (int));
    i = 1;
    j = 1;
    k = 1;
    for (m=0; m < n; m++) {
        idx[3*m]   = i;
        idx[3*m+1] = j;
        idx[3*m+2] = k++;
        if (k > n3){
            k = 1;
            j++;
            if (j > n2){
                j = 1;
                i++;
            }
        }
    }

    /* sort means in ascending order*/

    iSwitch=1;
    while (iSwitch != 0){
        iSwitch = 0;
        for (i = 0; i < n-1; i++){
            if (means[i] > means[i+1]){
                iSwitch = 1;
                x = means[i];
                means[i] = means[i+1];
                means[i+1] = x;
                j = idx[3*i];
                idx[3*i] = idx[3*(i+1)];
                idx[3*(i+1)] = j;
                j = idx[3*i+1];
                idx[3*i+1] = idx[3*(i+1)+1];
                idx[3*(i+1)+1] = j;
                j = idx[3*i+2];
                idx[3*i+2] = idx[3*(i+1)+2];
                idx[3*(i+1)+2] = j;
            }
        }
    }
    printf("[A][B][Split] \t  Mean \t\t  LSD Grouping \n");
    for (i=0; i < n; i++){
        printf("[%d][%d]  [%d] \t%f", idx[3*i], idx[3*i+1], idx[3*i+2],
            means[i]);

        for (j=1; j < i+1; j++){
            if(equalMeans[j-1] >= i+2-j){
                printf("\t*");
            }else{ 
                if(equalMeans[j-1]>0) printf("\t");
            }
        }
        if (i < n-1 && equalMeans[i]>0) printf("\t*");
        printf("\n");
    }
    imsls_free(idx);
    return;

}
